<?
/*  
 * Analysis Console for Intrusion Databases (ACID)
 *
 * Author: Roman Danyliw <rdd@cert.org>, <roman@danyliw.com>
 *
 * Copyright (C) 2000, 2001 Carnegie Mellon University
 * (see the file 'acid_main.php' for license details)
 *
 * Purpose: Displays the actual .GIF/.PNG/.TIFF image
 *          of the chart
 *
 * Input GET/POST variables
 *   - width: chart width
 *   - height: chart width
 *   - title: chart title
 *   - xaxis_label: x-axis label
 *   - yaxis_label: y-axis label
 *   - data[]: data array for the y-axis
 *   - datalbl[]: data label of x-axis
 *   - yaxis_scale: (boolean) 0: linear; 1: logorithmic
 *   - rotate_xaxis_lbl: (boolean) rotate X-axis labels 90 degrees
 *   - style: [bar|line] chooses the style of the chart
 */

  include ("acid_conf.php");
  include ("acid_graph_common.php");

  LoadGraphingLib($ChartLib_path);

  /* Verifies that the chart width is not too small
   *
   * algorithm: width >= const + number_of_points * factor
   */
  $width_const  = 50;
  $width_factor = 10;
  if ( $width < ($width_const + count ($data) * $width_factor)) {
    $width = $width_const + count ($data) * $width_factor;
  }

  $graph = new PHPlot($width, $height);
  $graph->SetIsInline("1");

  /* Create the data array */
  for ($i = 0; $i < count($data); $i++)
  {
      $xdata[$i] = array($datalbl[$i], $data[$i]);
  }

  $graph->SetDataType("text-data");
  if ( $style == "bar" )
  {
     $graph->SetPlotType("bars");
  }
  else if ( $style == "line" )
  {
     $graph->SetPlotType("linepoints");
  }

  /* read the X-axis labels as text */
  $graph->SetXGridLabelType ("title");

  $graph->SetPrecisionY(0);
  $graph->SetPrecisionX(0);

  $graph->SetBackgroundColor($chart_bg_color_default);
  $graph->SetLightGridColor($chart_lgrid_color_default);

  /* Check if need to rotate X-Axis label */
  if ( $rotate_xaxis_lbl == 1 )
     $graph->SetXDataLabelAngle(90);

  $graph->SetFileFormat($chart_file_format);
  //$graph->SetUseTTF("0");

  /* Set y-axis scale */
  if ( $yaxis_scale == 1 )
    $graph->SetYScaleType("log");

  $graph->SetDrawYGrid("1");
  $graph->SetDataValues($xdata);
  $graph->SetXLabel($xaxis_label);
  $graph->SetYLabel($yaxis_label);
  $graph->SetTitle($title);
  $graph->SetDataColors( array( $chart_bar_color_default ),
                         array("black") );
  //$graph->SetBackgroundColor("grey");
  $graph->DrawGraph();
?>







