<?php
/*  
 * Analysis Console for Incident Databases (ACID)
 *
 * Author: Roman Danyliw <rdd@cert.org>, <roman@danyliw.com>
 *
 * Copyright (C) 2000 Carnegie Mellon University
 * (see the file 'acid_main.php' for license details)
 *
 * Purpose:   
 *
 */

function SensorCnt($db)
{
   $result = $db->acidExecute("SELECT count(*) FROM sensor");
   $myrow = $result->acidFetchRow();
   $num = $myrow[0];
   $result->acidFreeRows();

   return $num;
}

function EventCnt($db)
{
   $result = $db->acidExecute("SELECT count(*) FROM event");
   $myrow = $result->acidFetchRow();
   $num = $myrow[0];
   $result->acidFreeRows();

   return $num;
}

/*
 * Takes: Numeric sensor ID from the Sensor table (SID), and 
 *	  database connection.
 * 
 * Returns: The number of unique alert descriptions for the 
 * 	    given sensor ID. 
 *
 */
function UniqueCntBySensor($sensorID, $db)
{

  /* Calculate the Unique Alerts */
  $query = "SELECT DISTINCT signature FROM event WHERE sid = " . $sensorID . ";";
  $result = $db->acidExecute($query);

  if ( $result ) 
  {
     $num = $result->acidRecordCount();
     $result->acidFreeRows();
  }
  else
     $num = 0;

  return $num;
}

/*
 * Takes: Numeric sensor ID from the Sensor table (SID), and 
 *        database connection.
 * 
 * Returns: The total number of alerts for the given sensor ID
 */ 
function EventCntBySensor($sensorID, $db)
{
   $query = "SELECT count(*) FROM event where SID = " .$sensorID. ";";

   $result = $db->acidExecute($query);
   $myrow = $result->acidFetchRow();
   $num = $myrow[0];
   $result->acidFreeRows();

   return $num;
}

function MinDateBySensor($sensorID, $db)
{
   $query = "SELECT min(timestamp) FROM event WHERE sid=" . $sensorID . ";";

   $result = $db->acidExecute($query);
   $myrow = $result->acidFetchRow();
   $num = $myrow[0];
   $result->acidFreeRows();

   return $num;
}


function MaxDateBySensor($sensorID, $db)
{
   $query = "SELECT max(timestamp) FROM event WHERE sid=" . $sensorID . ";";

   $result = $db->acidExecute($query);
   $myrow = $result->acidFetchRow();
   $num = $myrow[0];
   $result->acidFreeRows();

   return $num;
}

function UniqueDestAddrCntBySensor( $sensorID, $db )
{
   $query = "SELECT DISTINCT ip_dst from iphdr WHERE sid=" . $sensorID . ";";

   $result = $db->acidExecute($query);
   $num = $result->acidRecordCount();
   $result->acidFreeRows();

   return $num;
}

function UniqueSrcAddrCntBySensor( $sensorID, $db )
{
   $query = "SELECT DISTINCT ip_src from iphdr WHERE sid=" . $sensorID . ";";

   $result = $db->acidExecute($query);
   $num = $result->acidRecordCount();
   $result->acidFreeRows();

   return $num;
}

function TCPPktCnt($db)
{
   $result = $db->acidExecute("SELECT count(sid) FROM tcphdr;");
   $myrow = $result->acidFetchRow();
   $num = $myrow[0];
   $result->acidFreeRows();

   return $num;
}

function UDPPktCnt($db)
{
   $result = $db->acidExecute("SELECT count(sid) FROM udphdr;");
   $myrow = $result->acidFetchRow();
   $num = $myrow[0];
   $result->acidFreeRows();

   return $num;
}

function ICMPPktCnt($db)
{
   $result = $db->acidExecute("SELECT count(sid) FROM icmphdr;");
   $myrow = $result->acidFetchRow();
   $num = $myrow[0];
   $result->acidFreeRows();

   return $num;
}

function PortscanPktCnt($db)
{
  GLOBAL $DBtype;

  $schema_version = db_schema_version($db);

  /* Catch bug in ./create_postgresql v100 which defined event.signature as TEXT */
  if ( ($DBtype == "postgres") && ( $schema_version == 100 ) )
     $result = $db->acidExecute("SELECT count(event.sid) FROM event ".
                                "LEFT JOIN signature ON event.signature::int4=signature.sig_id ".
                                "WHERE sig_name LIKE 'spp_portscan%'");
  else if ( $schema_version >= 100 )
      $result = $db->acidExecute("SELECT count(event.sid) FROM event ".
                                 "LEFT JOIN signature ON event.signature=signature.sig_id ".
                                 "WHERE sig_name LIKE 'spp_portscan%'");
  else
      $result = $db->acidExecute("SELECT count(event.sid) FROM event ".
                                 "WHERE signature LIKE 'spp_portscan%'");


   $myrow = $result->acidFetchRow();
   $num = $myrow[0];
   $result->acidFreeRows();

   return $num;
}

function UniqueSrcIPCnt($db)
{
   $result = $db->acidExecute("SELECT DISTINCT ip_src FROM iphdr;");
   $num = $result->acidRecordCount();
   $result->acidFreeRows();

   return $num;
}

function UniqueDstIPCnt($db)
{
   $result = $db->acidExecute("SELECT DISTINCT ip_dst FROM iphdr;");
   $num = $result->acidRecordCount();
   $result->acidFreeRows();

   return $num;
}

function StartStopTime(&$start_time, &$stop_time, $db)
{
   $result = $db->acidExecute("SELECT min(timestamp), max(timestamp) FROM event");
   $myrow = $result->acidFetchRow();
   $start_time = $myrow[0];
   $stop_time = $myrow[1];
   $result->acidFreeRows();
}

function UniqueAlertCnt($db)
{
   $result = $db->acidExecute("SELECT DISTINCT signature FROM event");
   $num = $result->acidRecordCount();
   $result->acidFreeRows();

   return $num;
}

?>
